/*__________________________________________________________________

	Title: Spherical Image Deform
	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Author: Jolyon Troscianko
	Date: 16/1/2018
............................................................................................................

This script takes a square image and 'wraps' it around a spherical
object, as if it were a 3D ball.

The script applies linear interpolation for minimising artefacts.

To create a 3D-simulated shadow, use a circular selection centered
on the 'top' point of the sphere.

____________________________________________________________________
*/


import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;



public class Affine_align_slice implements PlugInFilter {

ImageStack stack;
	public int setup(String arg, ImagePlus imp) { 
	stack = imp.getStack(); 
	return DOES_32; 
	}

public void run(ImageProcessor ip) {

GenericDialog gd = new GenericDialog("GabRat Disruption Measurement");
		gd.addMessage("Affine Matrix Settings");
		gd.addNumericField("xC", 0, 6);
		gd.addNumericField("xX", 1, 6);
		gd.addNumericField("xY", 0, 6);
		gd.addNumericField("yC", 0, 6);
		gd.addNumericField("yX", 0, 6);
		gd.addNumericField("yY", 1, 6);
		gd.addNumericField("Slice", 1, 0);

	gd.showDialog();
	if (gd.wasCanceled())
		return;

double xC = gd.getNextNumber();
double xX = gd.getNextNumber();
double xY = gd.getNextNumber();

double yC = gd.getNextNumber();
double yX = gd.getNextNumber();
double yY = gd.getNextNumber();

int tSlice = (int) Math.round( gd.getNextNumber() );


int w = stack.getWidth();
int h = stack.getHeight();

float[] px =(float[]) stack.getPixels(tSlice);

float[] outPx = new float[w*h];



double xxd = 0.0;
double yyd = 0.0;

int xx = 0;
int yy = 0;
int xxp = 0;
int yyp = 0;


double xa = 0.0;
double xb = 0.0;


double ya = 0.0;
double yb = 0.0;

	

for(int y=0; y<h; y++)
for(int x=0; x<w; x++){

	xxd = xC +(x*xX)+(y*xY);
	yyd = yC +(x*yX)+(y*yY);


	xx = (int) (Math.round(xxd));
	yy = (int) (Math.round(yyd));

	xxd = xxd-xx;
	yyd = yyd-yy;

	if(xxd > 0)
		xxp = xx+1;
	else xxp = xx-1;

	if(yyd > 0)
		yyp = yy+1;
	else yyp = yy-1;

	if(    yy > 1 && xx > 1 && yy < h-1 && xx < w-1){

		xxd = Math.abs(xxd);
		yyd = Math.abs(yyd);
		
		xa = px[(yy*w)+xx] * (1-xxd);
		xb = px[(yy*w)+xxp] * xxd; // left/right

		ya = px[(yy*w)+xx] * (1-yyd);
		yb = px[(yyp*w)+xx] * yyd; // above/below


		outPx[(y*w)+x] =  (float) ((xa+xb+ya+yb)/2);
	}
} //xy


for(int i=0; i<w*h; i++)
	px[i] = outPx[i];



/*
ImageStack outStack = new ImageStack(w, h);
outStack.addSlice("Affine Transformed", outPx);
new ImagePlus("Output", outStack).show();
*/

}
}
