/*__________________________________________________________________

	Title: Average Stack
	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Author: Jolyon Troscianko
	Date: 10/3/2020
............................................................................................................

Simply takes the average of each pixel in a stack and outputs a single
image as the mean.

The image should be a stack with 32-bits/channel.

____________________________________________________________________
*/


import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;

public class Average_Stack implements PlugInFilter {

ImageStack stack;
	public int setup(String arg, ImagePlus imp) { 
	stack = imp.getStack(); 
	return DOES_32 + STACK_REQUIRED; 
	}
public void run(ImageProcessor ip) {

IJ.showStatus("Averaging Stack");

	int w = stack.getWidth();
	int h = stack.getHeight();
	int nz = stack.getSize();
	int d = w*h;

	float[] pixels= new float[d];
	float[] sums= new float[d];

	for(int i=0; i<nz; i++){
		pixels = (float[]) stack.getPixels(i+1);
		for(int j=0; j<d; j++)
			sums[j] = (float) (sums[j] + (pixels[j] / nz));
	}



ImageStack outStack = new ImageStack(w, h);
outStack.addSlice("Average", sums);
new ImagePlus("Stack Average", outStack).show();

}
}
