/*__________________________________________________________________

	Title: XYZ to sR'G'B'
	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Author: Jolyon Troscianko
	Date: 21/01/2020
............................................................................................................

Based on the reverse matrix here: https://www.image-engineering.de/library/technotes/958-how-to-convert-between-srgb-and-ciexyz

Note that these are the linear (R'G'B') values and need to go through gamma correction before being true sRGB values

____________________________________________________________________
*/


import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;

public class CIEXYZ_to_Linear_sRGB implements PlugInFilter {

ImageStack stack;
	public int setup(String arg, ImagePlus imp) { 
	stack = imp.getStack(); 
	return DOES_32 + STACK_REQUIRED; 
	}
public void run(ImageProcessor ip) {

float[] X;
float[] Y;
float[] Z;
int w = stack.getWidth();
int h = stack.getHeight();
int dimension = w*h;

float[] R = new float[dimension];
float[] G = new float[dimension];
float[] B = new float[dimension];

X = (float[]) stack.getPixels(1);
Y = (float[]) stack.getPixels(2);
Z = (float[]) stack.getPixels(3);

double xC = 0.816630208721392;
double yC = 1.03773203269915;
double zC = 1.10366746574307;


for (int i=0;i<dimension;i++) {

	double tX = X[i];
	double tY = Y[i];
	double tZ = Z[i];

	R[i] = (float) ((3.2404542*tX - 1.5371385*tY - 0.4985314*tZ ) * xC);
	G[i] = (float) ((-0.9692660*tX + 1.8760108*tY + 0.0415560*tZ ) * yC);
	B[i] = (float) ((0.0556434*tX - 0.2040259*tY + 1.0572252*tZ ) * zC);
}



ImageStack outStack = new ImageStack(w, h);
outStack.addSlice("R", R);
outStack.addSlice("G", G);
outStack.addSlice("B", B);
new ImagePlus("Linear sRGB", outStack).show();

}
}
